/****
 * CSignKeyWindow.c
 *
 *	Sign key window methods for the SignWriter key editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/

#include "CSignKeyWindow.h"
#include "Commands.h"
#include <Constants.h>
#include "CApplication.h"
#include "CBartender.h"
#include "CDesktop.h"
#include "CFileWndKeys.h"
#include "CSignKeyWndGrid.h"
#include "CSymWindow.h"
#include "CWindow.h"
#include "Global.h"
#include "Keys.h"
#include "KeyEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"





/*** Global Variables ***/


extern	CBartender	 *gBartender;	/* The menu handling object 			    */
extern	CDesktop	 *gDesktop;		/* The enclosure for all windows 			*/
extern	CBureaucrat	 *gGopher;		/* The current boss in the chain of command */

extern  CSymWindow   *gSymWind;		/* Sym window access */

extern  CFileWndKeys *gKeyPanes;	/* Key display access */

extern  void		 *gClipRec;		/* Clipboard */
extern  ObjType	 	  gClipType;




/***
 * ISignKeyWindow
 *
 *  Initialize a new instance of a sign key window.
 *
 ***/
 
void CSignKeyWindow::ISignKeyWindow (CDirectorOwner *aSupervisor,
									 Byte keyCode, char keyName)

{
	itsGridPane = NULL;
	itsKeyCode  = keyCode;
	itsKeyName  = keyName;
	
	inherited::IDirector(aSupervisor);
}



/***
 * Dispose
 *
 *
 ***/

void CSignKeyWindow::Dispose()

{
	inherited::Dispose();
}
			
			
					
/***
 * Dawdle
 *
 *
 ***/

void CSignKeyWindow::Dawdle (long *maxSleep)

{
	itsGridPane->BlinkSelect(maxSleep);
}


					
/***
 * DoCommand
 *
 *
 ***/

void CSignKeyWindow::DoCommand (long theCommand)

{
	switch (theCommand) {
			
	
		case cmdCut:
		
			itsGridPane->DoCopy();
			itsGridPane->DoClear(iCUT);
			break;
			
				
		case cmdCopy:
		
			itsGridPane->DoCopy();
			break;
			
			
		case cmdPaste:
		
			itsGridPane->DoPaste();
			break;
			
			
		case cmdClear:
		
			itsGridPane->DoClear(iCLEAR);
			break;
		
			
		case cmdAdd:
		
			itsGridPane->DoAdd();
			break;

			
		case cmdFill:
		
			itsGridPane->DoFill();
			break;	
			
			
		case cmdCursor:
		
			itsGridPane->DoCursor();
			break;	
		
			
		case cmdMirror:
		
			itsGridPane->DoMirror();
			break;
			
			
		case cmdChange:
		
			itsGridPane->DoChange();
			break;	
			
			
		case cmdRotate:
		
			itsGridPane->DoRotate();
			break;
				
				
		default:
		
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * DoKeyDown
 *
 *	Handle SignWriter symbol command keys.
 *
 ***/

void CSignKeyWindow::DoKeyDown (char theChar, Byte keyCode, EventRecord *macEvent)

{
	if (!(macEvent->modifiers & cmdKey)) 	
	 	itsGridPane->DoSymCommand(keyCode);

	else
		inherited::DoKeyDown(theChar, keyCode, macEvent);		

}
 


/***
 * UpdateMenus
 *
 *
***/

void CSignKeyWindow::UpdateMenus (void)

{ 	
 	// do inherited
  	inherited::UpdateMenus();
  	
  	
  	// Apple menu
  	
  	gBartender->EnableCmd(cmdAbout);
 	
 	
  	// file management
  	
  	gBartender->DisableCmd(cmdNew);
	gBartender->DisableCmd(cmdOpen);
	gBartender->DisableCmd(cmdPrint);
	gBartender->DisableCmd(cmdPageSetup);
	
 	gBartender->EnableCmd(cmdClose);
 	gBartender->EnableCmd(cmdSave);
 	gBartender->EnableCmd(cmdSaveAs);
 	
 	gBartender->EnableCmd(cmdUpdateSignKbd);
	gBartender->EnableCmd(cmdRevert);	


	if (gGopher == this) {
	
		// symbol editing	 
	
		if (gSymWind->SelectedSymbol() > 0)
			gBartender->EnableCmd(cmdAdd);
					
		if (itsGridPane->SelectionType() == sym) {
		
			gBartender->EnableCmd(cmdFill);
		 	gBartender->EnableCmd(cmdCursor);
		 	gBartender->EnableCmd(cmdMirror);
		 	gBartender->EnableCmd(cmdChange);
		 	gBartender->EnableCmd(cmdRotate);		 	
		}
		
		if (itsGridPane->SelectionType() != none) {
		
		 	gBartender->EnableCmd(cmdCut);
		 	gBartender->EnableCmd(cmdCopy);
		 	gBartender->EnableCmd(cmdClear);	
		}
			
 		if (gClipType == box || gClipType == sym)
 			gBartender->EnableCmd(cmdPaste);	
 					
	}	
	
}
		
					
			
/***
 * BuildWindow
 *
 *	Build new instance of a sign key window and
 *	display the data specified by the argument.
 *
 ***/
	
void CSignKeyWindow::BuildWindow (void)

{
	CSignKeyWndGrid	*theGridPane;
	
	Rect			wLoc;
	static short	wCount = 0;	
	Str15			winTitle = " ";

	
	   /**
		**	Get the (staggered) window location.
		**
		**/
		
	SetRect(&wLoc, SKWND_LEFT, SKWND_TOP, 
				   SKWND_LEFT + SKWND_WIDTH, SKWND_TOP + SKWND_HEIGHT);
				   
	OffsetRect(&wLoc, wCount * HORIZ_WOFFSET, wCount * VERT_WOFFSET);
	wCount = (wCount + 1) % MAX_WOFFSETS;
	
	
	   /**
		**	Make the (invisible) window.
		**
		**/
		
	itsWindow = new CWindow;
	itsWindow->INewWindow(&wLoc, FALSE, 4, FALSE, TRUE, gDesktop, this);
	
	
		/**
		 **  Set window title to key name
		 **
		 **/
		 
	winTitle[0] = itsKeyName;
	itsWindow->SetTitle(CtoPstr(winTitle));
	
	
		/**
		 **	theGridPane controls the grid array that appears
		 ** in the sign key window.  It is used to display and
		 ** edit the symbol definition of the selected key.
		 **
		 **/

	theGridPane = new CSignKeyWndGrid;
    theGridPane->ISignKeyWndGrid(itsWindow, this, itsKeyCode);
    						 
    itsGridPane = theGridPane;
	
	
		/**
		 **
		 ** The current class fields all sym window
		 ** commands.
		 ** 
		 **/
		 
	this->BecomeGopher(TRUE);
	
	
		/**
		 **	Link the sym window to its parent window.
		 **
		 **/
		 
	 gKeyPanes->SetSignKeyWindow(itsKeyCode, this);
	 
	 
		/**
		 **	Send the window a Select() message to make
		 **	it the active window.
		 **
		 **/
	
	itsWindow->Select();
	
}



/***
 * Close (override)
 *
 *	Update (& unlink from) file window before closing
 *
 ***/
 
Boolean	CSignKeyWindow::Close (Boolean quitting)

{
	itsGridPane->Close();
	
	if (gKeyPanes->itsCurKbd == sign)
		gKeyPanes->UpdateKey(itsKeyCode);
		
	gKeyPanes->SetSignKeyWindow(itsKeyCode, NULL);
	
	return inherited::Close(quitting);
	
}
